using AntiAircraftGun.DrawingObjects;
using AntiAircraftGun.MovementStrategy;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.Window;

namespace AntiAircraftGun
{
    public partial class FormAntiAirCraftGun : Form
    {
        private BaseDrawingAntiAirCraftGun _drawingAntiAirCraftGun;

        private AbstractStrategy? _abstractStrategy;

        public BaseDrawingAntiAirCraftGun? SelectedZenit { get; private set; }
        public FormAntiAirCraftGun()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedZenit = null;
        }

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawingAntiAirCraftGun == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawingAntiAirCraftGun.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawingAntiAirCraftGun.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawingAntiAirCraftGun.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawingAntiAirCraftGun.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }
        private void Draw()
        {
            if (_drawingAntiAirCraftGun == null)
            {
                return;
            }
            Bitmap bmp = new Bitmap(pictureBoxAntiAircraftGun.Width, pictureBoxAntiAircraftGun.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingAntiAirCraftGun.DrawTransport(gr);
            pictureBoxAntiAircraftGun.Image = bmp;
        }

        private void CreateAdvancedAintiAirCraftGun_Click(object sender, EventArgs e)
        {
            Random random = new Random();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)); //  
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)); // .   EntityAdvancedAntiAircraftGun
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
                bodyColor = dialog.Color;
            if (dialog.ShowDialog() == DialogResult.OK)
                dopColor = dialog.Color;
            _drawingAntiAirCraftGun = new AdvancedDrawingAntiAirCraftGun(
                random.Next(100, 300),   // 
                random.Next(1000, 3000), // 
                bodyColor, additionalColor, dopColor,
                Convert.ToBoolean(random.Next(2)), // Rocket
                Convert.ToBoolean(random.Next(2)),
                pictureBoxAntiAircraftGun.Width,
                pictureBoxAntiAircraftGun.Height
            );

            _drawingAntiAirCraftGun.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void CreateAntiAirCraftGun_Click(object sender, EventArgs e)
        {
            Random random = new Random();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)); //  
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)); // . 
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = dialog.Color;
            }
            _drawingAntiAirCraftGun = new BaseDrawingAntiAirCraftGun(
                random.Next(100, 300),   // 
                random.Next(1000, 3000), // 
                bodyColor, additionalColor,
                pictureBoxAntiAircraftGun.Width,
                pictureBoxAntiAircraftGun.Height
            );

            _drawingAntiAirCraftGun.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void ButtonStep_Click(object sender, EventArgs e)
        {
            if (_drawingAntiAirCraftGun == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
                DrawingObjectAntiAirCraftGun(_drawingAntiAirCraftGun), pictureBoxAntiAircraftGun.Width,
                pictureBoxAntiAircraftGun.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }

        }
        private void ButtonSelectZenit_Click(object sender, EventArgs e)
        {
            SelectedZenit = _drawingAntiAirCraftGun;
            DialogResult = DialogResult.OK;
        }


     }
}